@@ -15,9 +15,11 @@ try:
 except ImportError:
     _has_statsmodels = False
 
-from .utils import set_hls_values, iqr, _kde_support
+from .utils import iqr, _kde_support
 from .palettes import color_palette, blend_palette
-from .axisgrid import JointGrid
+
+
+__all__ = ["distplot", "kdeplot", "rugplot"]
 
 
 def _freedman_diaconis_bins(a):
@@ -100,7 +102,7 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
         :context: close-figs
 
         >>> import seaborn as sns, numpy as np
-        >>> sns.set(rc={"figure.figsize": (8, 4)}); np.random.seed(0)
+        >>> sns.set(); np.random.seed(0)
         >>> x = np.random.randn(100)
         >>> ax = sns.distplot(x)
 
@@ -306,9 +308,15 @@ def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
     alpha = kwargs.get("alpha", 0.25)
     if shade:
         if vertical:
-            ax.fill_betweenx(y, 1e-12, x, facecolor=color, alpha=alpha)
+            ax.fill_betweenx(y, 0, x, facecolor=color, alpha=alpha)
         else:
-            ax.fill_between(x, 1e-12, y, facecolor=color, alpha=alpha)
+            ax.fill_between(x, 0, y, facecolor=color, alpha=alpha)
+
+    # Set the density axis minimum to 0
+    if vertical:
+        ax.set_xlim(0, None)
+    else:
+        ax.set_ylim(0, None)
 
     # Draw the legend here
     if legend:
@@ -342,7 +350,7 @@ def _scipy_univariate_kde(data, bw, gridsize, cut, clip):
             warnings.warn(msg, UserWarning)
     if isinstance(bw, string_types):
         bw = "scotts" if bw == "scott" else bw
-        bw = getattr(kde, "%s_factor" % bw)()
+        bw = getattr(kde, "%s_factor" % bw)() * np.std(data)
     grid = _kde_support(data, bw, gridsize, cut, clip)
     y = kde(grid)
     return grid, y
@@ -350,7 +358,7 @@ def _scipy_univariate_kde(data, bw, gridsize, cut, clip):
 
 def _bivariate_kdeplot(x, y, filled, fill_lowest,
                        kernel, bw, gridsize, cut, clip,
-                       axlabel, ax, **kwargs):
+                       axlabel, cbar, cbar_ax, cbar_kws, ax, **kwargs):
     """Plot a joint KDE estimate as a bivariate contour plot."""
     # Determine the clipping
     if clip is None:
@@ -382,6 +390,10 @@ def _bivariate_kdeplot(x, y, filled, fill_lowest,
         cset.collections[0].set_alpha(0)
     kwargs["n_levels"] = n_levels
 
+    if cbar:
+        cbar_kws = {} if cbar_kws is None else cbar_kws
+        ax.figure.colorbar(cset, cbar_ax, ax, **cbar_kws)
+
     # Label the axes
     if hasattr(x, "name") and axlabel:
         ax.set_xlabel(x.name)
@@ -438,7 +450,8 @@ def _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip):
 
 def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
             bw="scott", gridsize=100, cut=3, clip=None, legend=True,
-            cumulative=False, shade_lowest=True, ax=None, **kwargs):
+            cumulative=False, shade_lowest=True, cbar=False, cbar_ax=None,
+            cbar_kws=None, ax=None, **kwargs):
     """Fit and plot a univariate or bivariate kernel density estimate.
 
     Parameters
@@ -474,8 +487,15 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
         relevant when drawing a univariate plot or when ``shade=False``.
         Setting this to ``False`` can be useful when you want multiple
         densities on the same Axes.
-    ax : matplotlib axis, optional
-        Axis to plot on, otherwise uses current axis.
+    cbar : bool, optional
+        If True and drawing a bivariate KDE plot, add a colorbar.
+    cbar_ax : matplotlib axes, optional
+        Existing axes to draw the colorbar onto, otherwise space is taken
+        from the main axes.
+    cbar_kws : dict, optional
+        Keyword arguments for ``fig.colorbar()``.
+    ax : matplotlib axes, optional
+        Axes to plot on, otherwise uses current axes.
     kwargs : key, value pairings
         Other keyword arguments are passed to ``plt.plot()`` or
         ``plt.contour{f}`` depending on whether a univariate or bivariate
@@ -554,6 +574,13 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
 
         >>> ax = sns.kdeplot(x, cut=0)
 
+    Add a colorbar for the contours:
+
+    .. plot::
+        :context: close-figs
+
+        >>> ax = sns.kdeplot(x, y, cbar=True)
+
     Plot two shaded bivariate densities:
 
     .. plot::
@@ -571,15 +598,23 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
     if ax is None:
         ax = plt.gca()
 
+    if isinstance(data, list):
+        data = np.asarray(data)
+
     data = data.astype(np.float64)
     if data2 is not None:
+        if isinstance(data2, list):
+            data2 = np.asarray(data2)
         data2 = data2.astype(np.float64)
 
+    warn = False
     bivariate = False
     if isinstance(data, np.ndarray) and np.ndim(data) > 1:
+        warn = True
         bivariate = True
         x, y = data.T
     elif isinstance(data, pd.DataFrame) and np.ndim(data) > 1:
+        warn = True
         bivariate = True
         x = data.iloc[:, 0].values
         y = data.iloc[:, 1].values
@@ -588,13 +623,19 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
         x = data
         y = data2
 
+    if warn:
+        warn_msg = ("Passing a 2D dataset for a bivariate plot is deprecated "
+                    "in favor of kdeplot(x, y), and it will cause an error in "
+                    "future versions. Please update your code.")
+        warnings.warn(warn_msg, UserWarning)
+
     if bivariate and cumulative:
         raise TypeError("Cumulative distribution plots are not"
                         "supported for bivariate distributions.")
     if bivariate:
         ax = _bivariate_kdeplot(x, y, shade, shade_lowest,
                                 kernel, bw, gridsize, cut, clip, legend,
-                                ax, **kwargs)
+                                cbar, cbar_ax, cbar_kws, ax, **kwargs)
     else:
         ax = _univariate_kdeplot(data, shade, vertical, kernel, bw,
                                  gridsize, cut, clip, legend, ax,
@@ -635,228 +676,3 @@ def rugplot(a, height=.05, axis="x", ax=None, **kwargs):
         func(pt, 0, height, **kwargs)
 
     return ax
-
-
-def jointplot(x, y, data=None, kind="scatter", stat_func=stats.pearsonr,
-              color=None, size=6, ratio=5, space=.2,
-              dropna=True, xlim=None, ylim=None,
-              joint_kws=None, marginal_kws=None, annot_kws=None, **kwargs):
-    """Draw a plot of two variables with bivariate and univariate graphs.
-
-    This function provides a convenient interface to the :class:`JointGrid`
-    class, with several canned plot kinds. This is intended to be a fairly
-    lightweight wrapper; if you need more flexibility, you should use
-    :class:`JointGrid` directly.
-
-    Parameters
-    ----------
-    x, y : strings or vectors
-        Data or names of variables in ``data``.
-    data : DataFrame, optional
-        DataFrame when ``x`` and ``y`` are variable names.
-    kind : { "scatter" | "reg" | "resid" | "kde" | "hex" }, optional
-        Kind of plot to draw.
-    stat_func : callable or None, optional
-        Function used to calculate a statistic about the relationship and
-        annotate the plot. Should map `x` and `y` either to a single value
-        or to a (value, p) tuple. Set to ``None`` if you don't want to
-        annotate the plot.
-    color : matplotlib color, optional
-        Color used for the plot elements.
-    size : numeric, optional
-        Size of the figure (it will be square).
-    ratio : numeric, optional
-        Ratio of joint axes size to marginal axes height.
-    space : numeric, optional
-        Space between the joint and marginal axes
-    dropna : bool, optional
-        If True, remove observations that are missing from ``x`` and ``y``.
-    {x, y}lim : two-tuples, optional
-        Axis limits to set before plotting.
-    {joint, marginal, annot}_kws : dicts, optional
-        Additional keyword arguments for the plot components.
-    kwargs : key, value pairings
-        Additional keyword arguments are passed to the function used to
-        draw the plot on the joint Axes, superseding items in the
-        ``joint_kws`` dictionary.
-
-    Returns
-    -------
-    grid : :class:`JointGrid`
-        :class:`JointGrid` object with the plot on it.
-
-    See Also
-    --------
-    JointGrid : The Grid class used for drawing this plot. Use it directly if
-                you need more flexibility.
-
-    Examples
-    --------
-
-    Draw a scatterplot with marginal histograms:
-
-    .. plot::
-        :context: close-figs
-
-        >>> import numpy as np, pandas as pd; np.random.seed(0)
-        >>> import seaborn as sns; sns.set(style="white", color_codes=True)
-        >>> tips = sns.load_dataset("tips")
-        >>> g = sns.jointplot(x="total_bill", y="tip", data=tips)
-
-    Add regression and kernel density fits:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.jointplot("total_bill", "tip", data=tips, kind="reg")
-
-    Replace the scatterplot with a joint histogram using hexagonal bins:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.jointplot("total_bill", "tip", data=tips, kind="hex")
-
-    Replace the scatterplots and histograms with density estimates and align
-    the marginal Axes tightly with the joint Axes:
-
-    .. plot::
-        :context: close-figs
-
-        >>> iris = sns.load_dataset("iris")
-        >>> g = sns.jointplot("sepal_width", "petal_length", data=iris,
-        ...                   kind="kde", space=0, color="g")
-
-    Use a different statistic for the annotation:
-
-    .. plot::
-        :context: close-figs
-
-        >>> from scipy.stats import spearmanr
-        >>> g = sns.jointplot("size", "total_bill", data=tips,
-        ...                   stat_func=spearmanr, color="m")
-
-    Draw a scatterplot, then add a joint density estimate:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = (sns.jointplot("sepal_length", "sepal_width",
-        ...                    data=iris, color="k")
-        ...         .plot_joint(sns.kdeplot, zorder=0, n_levels=6))
-
-    Pass vectors in directly without using Pandas, then name the axes:
-
-    .. plot::
-        :context: close-figs
-
-        >>> x, y = np.random.randn(2, 300)
-        >>> g = (sns.jointplot(x, y, kind="hex", stat_func=None)
-        ...         .set_axis_labels("x", "y"))
-
-    Draw a smaller figure with more space devoted to the marginal plots:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.jointplot("total_bill", "tip", data=tips,
-        ...                   size=5, ratio=3, color="g")
-
-    Pass keyword arguments down to the underlying plots:
-
-    .. plot::
-        :context: close-figs
-
-        >>> g = sns.jointplot("petal_length", "sepal_length", data=iris,
-        ...                   marginal_kws=dict(bins=15, rug=True),
-        ...                   annot_kws=dict(stat="r"),
-        ...                   s=40, edgecolor="w", linewidth=1)
-
-    """
-    # Set up empty default kwarg dicts
-    if joint_kws is None:
-        joint_kws = {}
-    joint_kws.update(kwargs)
-    if marginal_kws is None:
-        marginal_kws = {}
-    if annot_kws is None:
-        annot_kws = {}
-
-    # Make a colormap based off the plot color
-    if color is None:
-        color = color_palette()[0]
-    color_rgb = mpl.colors.colorConverter.to_rgb(color)
-    colors = [set_hls_values(color_rgb, l=l) for l in np.linspace(1, 0, 12)]
-    cmap = blend_palette(colors, as_cmap=True)
-
-    # Initialize the JointGrid object
-    grid = JointGrid(x, y, data, dropna=dropna,
-                     size=size, ratio=ratio, space=space,
-                     xlim=xlim, ylim=ylim)
-
-    # Plot the data using the grid
-    if kind == "scatter":
-
-        joint_kws.setdefault("color", color)
-        grid.plot_joint(plt.scatter, **joint_kws)
-
-        marginal_kws.setdefault("kde", False)
-        marginal_kws.setdefault("color", color)
-        grid.plot_marginals(distplot, **marginal_kws)
-
-    elif kind.startswith("hex"):
-
-        x_bins = _freedman_diaconis_bins(grid.x)
-        y_bins = _freedman_diaconis_bins(grid.y)
-        gridsize = int(np.mean([x_bins, y_bins]))
-
-        joint_kws.setdefault("gridsize", gridsize)
-        joint_kws.setdefault("cmap", cmap)
-        grid.plot_joint(plt.hexbin, **joint_kws)
-
-        marginal_kws.setdefault("kde", False)
-        marginal_kws.setdefault("color", color)
-        grid.plot_marginals(distplot, **marginal_kws)
-
-    elif kind.startswith("kde"):
-
-        joint_kws.setdefault("shade", True)
-        joint_kws.setdefault("cmap", cmap)
-        grid.plot_joint(kdeplot, **joint_kws)
-
-        marginal_kws.setdefault("shade", True)
-        marginal_kws.setdefault("color", color)
-        grid.plot_marginals(kdeplot, **marginal_kws)
-
-    elif kind.startswith("reg"):
-
-        from .linearmodels import regplot
-
-        marginal_kws.setdefault("color", color)
-        grid.plot_marginals(distplot, **marginal_kws)
-
-        joint_kws.setdefault("color", color)
-        grid.plot_joint(regplot, **joint_kws)
-
-    elif kind.startswith("resid"):
-
-        from .linearmodels import residplot
-
-        joint_kws.setdefault("color", color)
-        grid.plot_joint(residplot, **joint_kws)
-
-        x, y = grid.ax_joint.collections[0].get_offsets().T
-        marginal_kws.setdefault("color", color)
-        marginal_kws.setdefault("kde", False)
-        distplot(x, ax=grid.ax_marg_x, **marginal_kws)
-        distplot(y, vertical=True, fit=stats.norm, ax=grid.ax_marg_y,
-                 **marginal_kws)
-        stat_func = None
-    else:
-        msg = "kind must be either 'scatter', 'reg', 'resid', 'kde', or 'hex'"
-        raise ValueError(msg)
-
-    if stat_func is not None:
-        grid.annotate(stat_func, **annot_kws)
-
-    return grid
